/*
** Module   : SubMenuWidget.java
** Abstract : server side sub-menu widget implementation
**
** Copyright (c) 2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------------Description-----------------------------
** 001 VIG 20141124 Created initial version.
** 002 VIG 20150115 Changed base class, added initialization method.
*/
package com.goldencode.p2j.ui;

import java.lang.reflect.*;

import com.goldencode.p2j.util.*;

/**
 * This class represents sub-menu widget.
 */
@LegacyResource(resource = LegacyResource.SUB_MENU)
public class SubMenuWidget
extends MenuContainerWidget<SubMenuConfig>
{
   /**
    * Default constructor.  
    * 
    * @param    dynamic
    *           Flag indicating if this is a static or dynamic resource.
    */
   public SubMenuWidget(boolean dynamic)
   {
      super(dynamic, new SubMenuConfig());
   }

   /**
    * Gets this sub-menu parent (MENU or SUB-MENU).
    * 
    * @return  See above.
    */
   @Override
   public GenericWidget<?> getParent()
   {
      return LogicalTerminal.getWidgetForId(config.parentId);
   }

   /**
    * Sets this sub-menu parent (MENU or SUB-MENU).
    * 
    * @param  parent
    *         MENU or SUB-MENU widget.
    */
   @Override
   public void setParent(GenericWidget<?> parent)
   {
      if (parent instanceof MenuWidget || parent instanceof SubMenuWidget)
      {
         config.parentId = parent.getId();
      }
      else
      {
         // TODO "**Unable to assign UNKNOWN value to attribute PARENT on SUB-MENU widget. (4083)"
         String msgTemplate = "**%s widget cannot be a parent of a SUB-MENU widget";
         ErrorManager.recordOrThrowError(4075, String.format(msgTemplate, parent.type()), false);
      }
      
      pushScreenDefinition();
   }
   
   /**
    * Sets the label text, parsing mnemonic.
    * 
    * TODO: find out what is going on with undocumented feature of very long label
    * See parent's javadoc.
    * 
    * @param    label
    *           The new label text.
    */
   @Override
   public void setLabel(String label)
   {
      config.title = label;
      
      // TODO Check mnemonic support
   }
   
   /**
    * Finds menu-item in this menu tree. It can appear on different levels.
    * Menu-item is searched using some sort of depth-first algorithm.
    * See {@link MenuWidget#findMenuItem(String, MenuContainerWidget)}
    * 
    * @param   name
    *          MENU-ITEM name.
    *          
    * @return  concrete {@link MenuItemWidget} or null if not found.
    */
   public MenuItemWidget findMenuItem(String name)
   {
      return findMenuItemInt(name);
   }
   
   /**
    * Finds sub-menu in this sub-menu tree. It can appear only once. 
    * Simple depth-first search is used.
    * 
    * @param   name
    *          Sub-menu name.
    * 
    * @return  concrete {@link SubMenuWidget} or null if not found.
    */
   public SubMenuWidget findSubMenu(String name)
   {
      return findSubMenuInt(name);
   }
   
   /**
    * Creates new sub-menu as a proxy object (as a local resource).  This loads
    * the static sub-menu definition.
    *
    * @param    <T> 
    *           Sub-menu definition class type.
    * @param    defCls
    *           The class of sub-menu definition.
    * @param    name
    *           The name of sub-menu.          
    *
    * @return   The created sub-menu widget.
    */
   public static <T extends SubMenuDefinition> T createStaticSubMenu(Class<T> defCls, String name)
   {
      SubMenuWidget subMenu = new SubMenuWidget(false);
      subMenu.name(name);
      
      final T menuDef;
      try
      {
         menuDef = defCls.getConstructor(new Class<?>[]{}).newInstance();
         subMenu.config.id = new WidgetId(LogicalTerminal.registerMenu(subMenu));
         menuDef.setSelf(subMenu);
         menuDef.setup(subMenu);
         subMenu.coreInitialize(defCls, menuDef, false);
         subMenu.batch(false, true);
         //subMenu.finishConfigProcessing();
      }
      catch (NoSuchMethodException    | 
             SecurityException        | 
             InstantiationException   | 
             IllegalAccessException   | 
             IllegalArgumentException | 
             InvocationTargetException e)
      {
         throw new RuntimeException("Failed to instantiate sub-menu definition class", e);
      }
      
      return menuDef;
   }
   
   /**
    * Finds the first SUB-MENU with specified name in all menus.
    * 
    * @param   name
    *          The name of searched SUB-MENU
    *          
    * @return  The first found SubMenuWidget or null.
    */
   public static SubMenuWidget findSubMenuStatic(String name)
   {
      // TODO implement
      return null;
   }
   
}
